#include <stdio.h>
#include "sim.h"

extern void *malloc(size_t size);

#define POLL 1 /* adjtime frequency (per interval)*/
#define PVAL 1 /* print output frequeny (per poll) */
#define INTERVAL 1000000

struct {
	unsigned long long hardware_counter;
	double long simulator_time;
	int ppm;
} sim_data;


void simulator_data_init(long long offset, int ppm)
{
	sim_data.hardware_counter = 0;
	sim_data.simulator_time = offset;
	sim_data.ppm = ppm;
}

/* Get the currnet tick length */
u64 current_tick_length(long shift)
{
	u64 ret = INTERVAL;
	ret += ((double long)sim_data.ppm*INTERVAL)/1000000;
	return (ret <<shift);
}

void increment_simulator_time()
{
	/* increment the hardware and simulator time */
	sim_data.hardware_counter += INTERVAL;
	sim_data.simulator_time += current_tick_length(0);
}

unsigned long long simulator_get_hardware(void)
{
	return sim_data.hardware_counter;
}

unsigned long long simulator_get_time(void)
{
	return (unsigned long long)sim_data.simulator_time;
}


int main(int argc, void** argv)
{
	long long i;
	static int p;
	long long user_offset = 0;
	long user_ppm = 0;

	/* arg processing */
	if(argc > 1)
		user_ppm = strtol(argv[1],0,0);

	srand(time(0));
	/* setup the simulator values from the arguments*/
	simulator_data_init(0,  user_ppm);

	/* start the timeofday code */
	timekeeping_init();


	printf("simulator time:   kernel time    hardware cycle  time offset\n");
	printf("------------------------------------------------------------\n");
	while(1) {
		/* increment the simulator state and notify the kernel*/
		increment_simulator_time();
		if (rand() % 8)
			update_wall_time();

		/* every so often, do adjtimex */
		if(!(i++%POLL)) {
			unsigned long long sim_time, kern_time, hardware;
			long long diff; 
			struct timespec t;
			/* get the current simulator/kernel state */
			sim_time = simulator_get_time();
			hardware = simulator_get_hardware();
			getnstimeofday(&t);
			kern_time = (u64)t.tv_sec*NSEC_PER_SEC + t.tv_nsec;

			/* calculate the offset */
			diff = sim_time - kern_time;

			/* print simulator state every so often */
			if (!(p++%PVAL))
				printf("%llu ns:  %llu ns  %llu cyc  %lld ns \n", sim_time, kern_time, hardware, diff);
		}
	}	
	return 0;
}



